@@ -1,5 +1,5 @@
 package Crypt::PKCS11::Easy;
-$Crypt::PKCS11::Easy::VERSION = '0.161160';
+$Crypt::PKCS11::Easy::VERSION = '0.161250';
 # ABSTRACT: Wrapper around Crypt::PKCS11 to make using a HSM not suck
 
 use v5.16.3;    # CentOS7
@@ -7,9 +7,11 @@ use Crypt::PKCS11 qw/:constant_names :constant/;
 use Crypt::PKCS11::Attributes;
 use Log::Any '$log';
 use Path::Tiny;
+use Safe::Isa;
 use Try::Tiny;
+use Types::Standard qw/ArrayRef Str/;
+use Types::Path::Tiny 'AbsFile';
 use version;
-use Sub::Quote;
 use Moo;
 use namespace::clean;
 
@@ -19,7 +21,36 @@ use experimental 'smartmatch';
 has module => (
     is       => 'ro',
     required => 1,
-    isa => quote_sub(q{ die 'module is not defined' unless defined $_[0] }),
+    isa      => Str,
+);
+
+has _module => (
+    is      => 'ro',
+    lazy    => 1,
+    default => sub {
+        my $self = shift;
+
+        # is module already a Path::Tiny object?
+        return $self->module if $self->module->$_isa('Path::Tiny');
+
+        # does module look like a path?
+        return path($self->module)->absolute if $self->module =~ m|/|;
+
+        # TODO care about non-linux?
+        # just a string, lets try to find a module
+        my $module_name = sprintf '%s.so', $self->module;
+        my $full_module_path;
+        for (@{$self->_module_dirs}) {
+            next unless $_->child($module_name)->is_file;
+            $full_module_path = $_->child($module_name);
+        }
+        if (!$full_module_path) {
+            die 'Unable to find a module for ' . $self->module;
+        }
+        return $full_module_path;
+    },
+
+    isa => AbsFile,
 );
 
 
@@ -40,6 +71,14 @@ has token => (is => 'ro', predicate => 1);
 
 has pin => (is => 'ro', required => 0);
 
+
+has module_dirs => (
+    is      => 'ro',
+    lazy    => 1,
+    isa     => ArrayRef,
+    default => sub { ['/usr/lib64/pkcs11/'] },
+);
+
 has _pkcs11 => (is => 'rwp');
 
 has _key => (is => 'lazy');
@@ -59,15 +98,20 @@ has _default_mech => (
     },
 );
 
-has _module_dir => (
+has _module_dirs => (
     is      => 'ro',
+    lazy    => 1,
     default => sub {
-
-        # TODO non-64 bit, an array of alternatives, override...
-        my $md = path '/usr/lib64/pkcs11/';
-        die "Invalid pkcs11 module dir $md" unless $md->is_dir;
-        return $md;
-    });
+        my $self = shift;
+        my @paths;
+        for (@{$self->module_dirs}) {
+            my $path = path($_)->absolute;
+            push @paths, $path if $path->is_dir;
+        }
+        die "No valid module paths found\n" if scalar @paths == 0;
+        return \@paths;
+    },
+);
 
 has _flags => (
     is      => 'ro',
@@ -126,22 +170,18 @@ sub _build__pkcs11 {
 
     $log->debug('Initialising PKCS#11...');
 
-    # TODO care about non-linux...?
-    my $module_name = sprintf '%s.so', $self->module;
-    my $module_path = $self->_module_dir->child($module_name);
-    die "Unable to find $module_path" unless $module_path->is_file;
-
     # Create the main PKCS #11 object, load a PKCS #11 provider .so library and initialize the module
     my $pkcs11 = Crypt::PKCS11->new;
 
-    $pkcs11->load($module_path)
-      or die "Failed to load PKCS11 module $module_path: " . $pkcs11->errstr;
+    $pkcs11->load($self->_module)
+      or die sprintf "Failed to load PKCS11 module [%s]: %s\n",
+      $self->_module, $pkcs11->errstr;
 
     $pkcs11->Initialize
-      or die "Failed to initialize PKCS11 module $module_path: "
-      . $pkcs11->errstr;
+      or die sprintf "Failed to initialize PKCS11 module [%s]: %s\n",
+      $self->_module, $pkcs11->errstr;
 
-    $log->debug("Loaded PKCS#11 module: $module_name");
+    $log->debug("Loaded PKCS#11 module: " . $self->_module);
 
     return $pkcs11;
 }
@@ -605,7 +645,7 @@ Crypt::PKCS11::Easy - Wrapper around Crypt::PKCS11 to make using a HSM not suck
 
 =head1 VERSION
 
-version 0.161160
+version 0.161250
 
 =head1 SYNOPSIS
 
@@ -693,8 +733,9 @@ created for each function.
 
 String. Required.
 
-The name of the PKCS#11 module to use. Just use the base name of the library
-and the rest will be handled automagically. e.g.
+The name of the PKCS#11 module to use. Either pass the full path to the module,
+or just pass the base name of the library and the rest will be handled
+automagically. e.g.
 
   libsofthsm2          => /usr/lib64/pkcs11/libsofthsm2.so
   libCryptoki2_64      => /usr/lib64/pkcs11/libCryptoki2_64.so
@@ -739,6 +780,10 @@ to access a token, a coderef that returns it, or a file that contains it.
 
  $pin = '1234';
 
+=head2 C<module_dirs>
+
+Array of paths to check for PKCS#11 modules.
+
 =head1 METHODS
 
 =head2 C<get_info>
@@ -847,6 +892,12 @@ Will return a hashref of available mechanisms. If a slot id is specifed, the
 mechanisms for that slot will be retrieved. Otherwise, the slot id in L</slot>
  will be used if there is one.
 
+=head1 STATUS
+
+=for HTML <div>
+    <a href="https://travis-ci.org/sophos/Crypt-PKCS11-Easy"><img src="https://travis-ci.org/sophos/Crypt-PKCS11-Easy.svg?branch=master"></a>
+</div>
+
 =head1 DIAGNOSTICS
 
 C<Crypt::PKCS11::Easy> uses L<Log::Any> for logging. To see debug output on
